@@ -3,6 +3,8 @@
 /* This representation's function pointer table. */
 static const MVMREPROps CStruct_this_repr;
 
+static void free_repr_data(MVMCStructREPRData *repr_data);
+
 /* Locates all of the attributes. Puts them onto a flattened, ordered
  * list of attributes (populating the passed flat_list). Also builds
  * the index mapping for doing named lookups. Note index is not related
@@ -138,6 +140,7 @@ static void compute_allocation_strategy(MVMThreadContext *tc, MVMObject *repr_in
 
         if (info_alloc == 0) {
             MVM_gc_allocate_gen2_default_clear(tc);
+            free_repr_data(repr_data);
             MVM_exception_throw_adhoc(tc, "Class %s has no attributes, which is illegal with the CStruct representation.", MVM_6model_get_stable_debug_name(tc, st));
         }
 
@@ -166,6 +169,7 @@ static void compute_allocation_strategy(MVMThreadContext *tc, MVMObject *repr_in
 
             if (num_dimensions > 1) {
                 MVM_gc_allocate_gen2_default_clear(tc);
+                free_repr_data(repr_data);
                 MVM_exception_throw_adhoc(tc,
                     "Only one dimensions supported in CStruct attribute");
             }
@@ -176,6 +180,7 @@ static void compute_allocation_strategy(MVMThreadContext *tc, MVMObject *repr_in
                 MVMint32  type_id    = REPR(type)->ID;
                 if (spec->inlineable == MVM_STORAGE_SPEC_INLINED &&
                         (spec->boxed_primitive == MVM_STORAGE_SPEC_BP_INT ||
+                         spec->boxed_primitive == MVM_STORAGE_SPEC_BP_UINT64 ||
                          spec->boxed_primitive == MVM_STORAGE_SPEC_BP_NUM)) {
                     /* It's a boxed int or num; pretty easy. It'll just live in the
                      * body of the struct. Instead of masking in i here (which
@@ -217,6 +222,7 @@ static void compute_allocation_strategy(MVMThreadContext *tc, MVMObject *repr_in
                         MVMCArrayREPRData *carray_repr_data = (MVMCArrayREPRData *)STABLE(type)->REPR_data;
                         if (!carray_repr_data) {
                             MVM_gc_allocate_gen2_default_clear(tc);
+                            free_repr_data(repr_data);
                             MVM_exception_throw_adhoc(tc,
                                 "CStruct: can't inline a CArray attribute before its type's definition");
                         }
@@ -255,6 +261,7 @@ static void compute_allocation_strategy(MVMThreadContext *tc, MVMObject *repr_in
                         MVMCStructREPRData *cstruct_repr_data = (MVMCStructREPRData *)STABLE(type)->REPR_data;
                         if (!cstruct_repr_data) {
                             MVM_gc_allocate_gen2_default_clear(tc);
+                            free_repr_data(repr_data);
                             MVM_exception_throw_adhoc(tc,
                                 "CStruct: can't inline a CStruct attribute before its type's definition");
                         }
@@ -272,6 +279,7 @@ static void compute_allocation_strategy(MVMThreadContext *tc, MVMObject *repr_in
                         MVMCPPStructREPRData *cppstruct_repr_data = (MVMCPPStructREPRData *)STABLE(type)->REPR_data;
                         if (!cppstruct_repr_data) {
                             MVM_gc_allocate_gen2_default_clear(tc);
+                            free_repr_data(repr_data);
                             MVM_exception_throw_adhoc(tc,
                                 "CStruct: can't inline a CPPStruct attribute before its type's definition");
                         }
@@ -289,6 +297,7 @@ static void compute_allocation_strategy(MVMThreadContext *tc, MVMObject *repr_in
                         MVMCUnionREPRData *cunion_repr_data = (MVMCUnionREPRData *)STABLE(type)->REPR_data;
                         if (!cunion_repr_data) {
                             MVM_gc_allocate_gen2_default_clear(tc);
+                            free_repr_data(repr_data);
                             MVM_exception_throw_adhoc(tc,
                                 "CStruct: can't inline a CUnion attribute before its type's definition");
                         }
@@ -305,6 +314,7 @@ static void compute_allocation_strategy(MVMThreadContext *tc, MVMObject *repr_in
                 }
                 else {
                     MVM_gc_allocate_gen2_default_clear(tc);
+                    free_repr_data(repr_data);
                     MVM_exception_throw_adhoc(tc,
                         "CStruct representation only handles attributes of type:\n"
                         "  (u)int8, (u)int16, (u)int32, (u)int64, (u)long, (u)longlong, num32, num64, (s)size_t, bool, Str\n"
@@ -313,13 +323,15 @@ static void compute_allocation_strategy(MVMThreadContext *tc, MVMObject *repr_in
             }
             else {
                 MVM_gc_allocate_gen2_default_clear(tc);
+                free_repr_data(repr_data);
                 MVM_exception_throw_adhoc(tc,
                     "CStruct representation requires the types of all attributes to be specified");
             }
 
             if (bits % 8) {
                 MVM_gc_allocate_gen2_default_clear(tc);
-                 MVM_exception_throw_adhoc(tc,
+                free_repr_data(repr_data);
+                MVM_exception_throw_adhoc(tc,
                     "CStruct only supports native types that are a multiple of 8 bits wide (was passed: %"PRId32")", bits);
             }
 
@@ -396,8 +408,8 @@ static MVMObject * type_object_for(MVMThreadContext *tc, MVMObject *HOW) {
 /* Composes the representation. */
 static void compose(MVMThreadContext *tc, MVMSTable *st, MVMObject *repr_info) {
     /* Compute allocation strategy. */
-    MVMCStructREPRData *repr_data = MVM_calloc(1, sizeof(MVMCStructREPRData));
     MVMObject *attr_info = MVM_repr_at_key_o(tc, repr_info, tc->instance->str_consts.attribute);
+    MVMCStructREPRData *repr_data = MVM_calloc(1, sizeof(MVMCStructREPRData));
     MVM_gc_allocate_gen2_default_set(tc);
     compute_allocation_strategy(tc, attr_info, repr_data, st);
     MVM_gc_allocate_gen2_default_clear(tc);
@@ -410,7 +422,7 @@ static void initialize(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, voi
 
     /* Allocate object body. */
     MVMCStructBody *body = (MVMCStructBody *)data;
-    body->cstruct = MVM_calloc(1, repr_data->struct_size > 0 ? repr_data->struct_size : 1);
+    body->cstruct = calloc(1, repr_data->struct_size > 0 ? repr_data->struct_size : 1);
 
     /* Allocate child obj array. */
     if (repr_data->num_child_objs > 0)
@@ -542,6 +554,14 @@ static void get_attribute(MVMThreadContext *tc, MVMSTable *st, MVMObject *root,
                 MVM_exception_throw_adhoc(tc, "CStruct: invalid native get of object attribute");
             break;
         }
+        case MVM_reg_uint64: {
+            if (attr_st)
+                result_reg->u64 = attr_st->REPR->box_funcs.get_uint(tc, attr_st, root,
+                    ((char *)body->cstruct) + repr_data->struct_offsets[slot]);
+            else
+                MVM_exception_throw_adhoc(tc, "CStruct: invalid native get of object attribute");
+            break;
+        }
         case MVM_reg_num64: {
             if (attr_st)
                 result_reg->n64 = attr_st->REPR->box_funcs.get_num(tc, attr_st, root,
@@ -665,6 +685,14 @@ static void bind_attribute(MVMThreadContext *tc, MVMSTable *st, MVMObject *root,
                 MVM_exception_throw_adhoc(tc, "CStruct: invalid native binding to object attribute");
             break;
         }
+        case MVM_reg_uint64: {
+            if (attr_st)
+                attr_st->REPR->box_funcs.set_uint(tc, attr_st, root,
+                    ((char *)body->cstruct) + repr_data->struct_offsets[slot], value_reg.u64);
+            else
+                MVM_exception_throw_adhoc(tc, "CStruct: invalid native binding to object attribute");
+            break;
+        }
         case MVM_reg_num64: {
             if (attr_st)
                 attr_st->REPR->box_funcs.set_num(tc, attr_st, root,
@@ -737,10 +765,7 @@ static void gc_mark_repr_data(MVMThreadContext *tc, MVMSTable *st, MVMGCWorklist
     }
 }
 
-/* Free representation data. */
-static void gc_free_repr_data(MVMThreadContext *tc, MVMSTable *st) {
-    MVMCStructREPRData *repr_data = (MVMCStructREPRData *)st->REPR_data;
-
+static void free_repr_data(MVMCStructREPRData *repr_data) {
     /* May not have survived to composition. */
     if (repr_data == NULL)
         return;
@@ -754,7 +779,13 @@ static void gc_free_repr_data(MVMThreadContext *tc, MVMSTable *st) {
         MVM_free(repr_data->initialize_slots);
     }
 
-    MVM_free(st->REPR_data);
+    MVM_free(repr_data);
+}
+
+/* Free representation data. */
+static void gc_free_repr_data(MVMThreadContext *tc, MVMSTable *st) {
+    MVMCStructREPRData *repr_data = (MVMCStructREPRData *)st->REPR_data;
+    free_repr_data(repr_data);
 }
 
 /* This is called to do any cleanup of resources when an object gets
@@ -766,7 +797,7 @@ static void gc_cleanup(MVMThreadContext *tc, MVMSTable *st, void *data) {
     /* XXX For some reason, this causes crashes at the moment. Need to
      * work out why. */
     /*if (body->cstruct)
-        MVM_free(body->cstruct);*/
+        free(body->cstruct);*/
 }
 
 /* Called by the VM in order to free memory associated with this object. */
@@ -789,7 +820,7 @@ static MVMuint64 unmanaged_size(MVMThreadContext *tc, MVMSTable *st, void *data)
     MVMCStructREPRData *repr_data = (MVMCStructREPRData *)st->REPR_data;
     MVMuint64 result = 0;
 
-    /* The allocated (or just-poisted-at) memory block */
+    /* The allocated (or just-pointed-at) memory block */
     /* TODO make sure when structs properly track "ownership" to
      *      not add this for "child" structs */
     result += repr_data->struct_size;
@@ -851,9 +882,7 @@ static void deserialize_repr_data(MVMThreadContext *tc, MVMSTable *st, MVMSerial
     MVMint32 i, num_classes, num_slots;
 
     repr_data->struct_size = MVM_serialization_read_int(tc, reader);
-    if (reader->root.version >= 17) {
-        repr_data->struct_align = MVM_serialization_read_int(tc, reader);
-    }
+    repr_data->struct_align = MVM_serialization_read_int(tc, reader);
     repr_data->num_attributes = MVM_serialization_read_int(tc, reader);
     repr_data->num_child_objs = MVM_serialization_read_int(tc, reader);
 
