@@ -21,79 +21,93 @@ static MVMObject * type_object_for(MVMThreadContext *tc, MVMObject *HOW) {
 static void copy_to(MVMThreadContext *tc, MVMSTable *st, void *src, MVMObject *dest_root, void *dest) {
     MVMHashAttrStoreBody *src_body  = (MVMHashAttrStoreBody *)src;
     MVMHashAttrStoreBody *dest_body = (MVMHashAttrStoreBody *)dest;
-    MVMHashEntry *current;
 
-    /* NOTE: if we really wanted to, we could avoid rehashing... */
-    HASH_ITER_FAST(tc, hash_handle, src_body->hash_head, current, {
-        MVMHashEntry *new_entry = MVM_malloc(sizeof(MVMHashEntry));
-        MVM_HASH_BIND_FREE(tc, dest_body->hash_head, MVM_HASH_KEY(current), new_entry, {
-            MVM_free(new_entry);
-        });
-        MVM_ASSIGN_REF(tc, &(dest_root->header), new_entry->value, current->value);
-    });
+    MVMStrHashTable *src_hashtable = &(src_body->hashtable);
+    MVMStrHashTable *dest_hashtable = &(dest_body->hashtable);
+    if (MVM_str_hash_entry_size(tc, dest_hashtable)) {
+        /* copy_to is, on reference types, only ever used as part of clone, and
+         * that will always target a freshly created object.
+         * So this should be unreachable. */
+        MVM_oops(tc, "copy_to on MVMHash that is already initialized");
+    }
+    MVM_str_hash_build(tc, dest_hashtable, sizeof(MVMHashEntry),
+                       MVM_str_hash_count(tc, src_hashtable));
+    MVMStrHashIterator iterator = MVM_str_hash_first(tc, src_hashtable);
+    while (!MVM_str_hash_at_end(tc, src_hashtable, iterator)) {
+        MVMHashEntry *entry = MVM_str_hash_current_nocheck(tc, src_hashtable, iterator);
+        MVMHashEntry *new_entry = MVM_str_hash_insert_nocheck(tc, dest_hashtable, entry->hash_handle.key);
+        MVM_ASSIGN_REF(tc, &(dest_root->header), new_entry->value, entry->value);
+        MVM_gc_write_barrier(tc, &(dest_root->header), &(new_entry->hash_handle.key->common.header));
+        iterator = MVM_str_hash_next_nocheck(tc, src_hashtable, iterator);
+    }
 }
 
 /* Adds held objects to the GC worklist. */
 static void gc_mark(MVMThreadContext *tc, MVMSTable *st, void *data, MVMGCWorklist *worklist) {
     MVMHashAttrStoreBody *body = (MVMHashAttrStoreBody *)data;
-    MVMHashEntry *current;
+    MVMStrHashTable *hashtable = &(body->hashtable);
+    MVM_gc_worklist_presize_for(tc, worklist, 2 * MVM_str_hash_count(tc, hashtable));
 
-    HASH_ITER_FAST(tc, hash_handle, body->hash_head, current, {
+    MVMStrHashIterator iterator = MVM_str_hash_first(tc, hashtable);
+    while (!MVM_str_hash_at_end(tc, hashtable, iterator)) {
+        MVMHashEntry *current = MVM_str_hash_current_nocheck(tc, hashtable, iterator);
         MVM_gc_worklist_add(tc, worklist, &current->hash_handle.key);
         MVM_gc_worklist_add(tc, worklist, &current->value);
-    });
+        iterator = MVM_str_hash_next_nocheck(tc, hashtable, iterator);
+    }
 }
 
 /* Called by the VM in order to free memory associated with this object. */
 static void gc_free(MVMThreadContext *tc, MVMObject *obj) {
     MVMHashAttrStore *h = (MVMHashAttrStore *)obj;
-    MVM_HASH_DESTROY(tc, hash_handle, MVMHashEntry, h->body.hash_head);
+    MVMStrHashTable *hashtable = &(h->body.hashtable);
+
+    MVM_str_hash_demolish(tc, hashtable);
 }
 
 static void get_attribute(MVMThreadContext *tc, MVMSTable *st, MVMObject *root,
         void *data, MVMObject *class_handle, MVMString *name, MVMint64 hint,
         MVMRegister *result_reg, MVMuint16 kind) {
     MVMHashAttrStoreBody *body = (MVMHashAttrStoreBody *)data;
-    if (kind == MVM_reg_obj) {
-        MVMHashEntry *entry;
-        MVM_HASH_GET(tc, body->hash_head, name, entry);
-        result_reg->o = entry != NULL ? entry->value : tc->instance->VMNull;
-    }
-    else {
+    MVMStrHashTable *hashtable = &(body->hashtable);
+
+    if (MVM_UNLIKELY(kind != MVM_reg_obj))
         MVM_exception_throw_adhoc(tc,
             "HashAttrStore representation does not support native attribute storage");
-    }
+
+    MVMHashEntry *entry = MVM_str_hash_fetch(tc, hashtable, name);
+    result_reg->o = entry != NULL ? entry->value : tc->instance->VMNull;
 }
 
 static void bind_attribute(MVMThreadContext *tc, MVMSTable *st, MVMObject *root,
         void *data, MVMObject *class_handle, MVMString *name, MVMint64 hint,
         MVMRegister value_reg, MVMuint16 kind) {
     MVMHashAttrStoreBody *body = (MVMHashAttrStoreBody *)data;
-    if (kind == MVM_reg_obj) {
-        MVMHashEntry *entry;
-        MVM_HASH_GET(tc, body->hash_head, name, entry);
-        if (!entry) {
-            entry = MVM_malloc(sizeof(MVMHashEntry));
-            MVM_HASH_BIND_FREE(tc, body->hash_head, name, entry, {
-                MVM_free(entry);
-            });
-            MVM_ASSIGN_REF(tc, &(root->header), entry->value, value_reg.o);
-            MVM_gc_write_barrier(tc, &(root->header), &(name->common.header));
-        }
-        else {
-            MVM_ASSIGN_REF(tc, &(root->header), entry->value, value_reg.o);
-        }
+    MVMStrHashTable *hashtable = &(body->hashtable);
+
+    if (!MVM_str_hash_key_is_valid(tc, name)) {
+        MVM_str_hash_key_throw_invalid(tc, name);
     }
-    else {
+    if (MVM_UNLIKELY(kind != MVM_reg_obj))
         MVM_exception_throw_adhoc(tc,
             "HashAttrStore representation does not support native attribute storage");
+
+    if (!MVM_str_hash_entry_size(tc, hashtable)) {
+        MVM_str_hash_build(tc, hashtable, sizeof(MVMHashEntry), 0);
+    }
+
+    MVMHashEntry *entry = MVM_str_hash_lvalue_fetch_nocheck(tc, hashtable, name);
+    MVM_ASSIGN_REF(tc, &(root->header), entry->value, value_reg.o);
+    if (!entry->hash_handle.key) {
+        entry->hash_handle.key = name;
+        MVM_gc_write_barrier(tc, &(root->header), &(name->common.header));
     }
 }
 
 static MVMint64 is_attribute_initialized(MVMThreadContext *tc, MVMSTable *st, void *data, MVMObject *class_handle, MVMString *name, MVMint64 hint) {
     MVMHashAttrStoreBody *body = (MVMHashAttrStoreBody *)data;
-    MVMHashEntry *entry;
-    MVM_HASH_GET(tc, body->hash_head, name, entry);
+    MVMStrHashTable *hashtable = &(body->hashtable);
+    MVMHashEntry *entry = MVM_str_hash_fetch(tc, hashtable, name);
     return entry != NULL;
 }
 
@@ -127,6 +141,12 @@ static void deserialize_stable_size(MVMThreadContext *tc, MVMSTable *st, MVMSeri
     st->size = sizeof(MVMHashAttrStore);
 }
 
+static MVMuint64 unmanaged_size(MVMThreadContext *tc, MVMSTable *st, void *data) {
+    MVMHashBody *body = (MVMHashBody *)data;
+
+    return MVM_str_hash_allocated_size(tc, &(body->hashtable));
+}
+
 /* Initializes the representation. */
 const MVMREPROps * MVMHashAttrStore_initialize(MVMThreadContext *tc) {
     return &HashAttrStore_this_repr;
@@ -164,6 +184,6 @@ static const MVMREPROps HashAttrStore_this_repr = {
     NULL, /* spesh */
     "HashAttrStore", /* name */
     MVM_REPR_ID_HashAttrStore,
-    NULL, /* unmanaged_size */
+    unmanaged_size,
     NULL, /* describe_refs */
 };
