@@ -26,7 +26,7 @@ static void copy_to(MVMThreadContext *tc, MVMSTable *st, void *src, MVMObject *d
         : MVM_callsite_copy(tc, src_body->callsite);
     size_t arg_size = dest_body->callsite->flag_count * sizeof(MVMRegister);
     if (arg_size) {
-        dest_body->args = MVM_fixed_size_alloc(tc, tc->instance->fsa, arg_size);
+        dest_body->args = MVM_malloc(arg_size);
         memcpy(dest_body->args, src_body->args, arg_size);
     }
     else {
@@ -51,9 +51,7 @@ static void gc_mark(MVMThreadContext *tc, MVMSTable *st, void *data, MVMGCWorkli
 static void gc_free(MVMThreadContext *tc, MVMObject *obj) {
     MVMCapture *capture = (MVMCapture *)obj;
     if (capture->body.args)
-        MVM_fixed_size_free(tc, tc->instance->fsa,
-                capture->body.callsite->flag_count * sizeof(MVMRegister),
-                capture->body.args);
+        MVM_free(capture->body.args);
     if (capture->body.callsite && !capture->body.callsite->is_interned)
         MVM_callsite_destroy(capture->body.callsite);
 }
@@ -124,8 +122,7 @@ MVMObject * MVM_capture_from_args(MVMThreadContext *tc, MVMArgs arg_info) {
     MVMCallsite *callsite = arg_info.callsite;
     MVMRegister *args;
     if (callsite->flag_count) {
-        args = MVM_fixed_size_alloc(tc, tc->instance->fsa,
-                callsite->flag_count * sizeof(MVMRegister));
+        args = MVM_malloc(callsite->flag_count * sizeof(MVMRegister));
         MVMuint16 i;
         for (i = 0; i < callsite->flag_count; i++)
             args[i] = arg_info.source[arg_info.map[i]];
@@ -178,6 +175,8 @@ static MVMint64 flag_to_spec(MVMint64 flag) {
     switch (flag & MVM_CALLSITE_ARG_TYPE_MASK) {
         case MVM_CALLSITE_ARG_INT:
             return MVM_STORAGE_SPEC_BP_INT;
+        case MVM_CALLSITE_ARG_UINT:
+            return MVM_STORAGE_SPEC_BP_UINT64;
         case MVM_CALLSITE_ARG_NUM:
             return MVM_STORAGE_SPEC_BP_NUM;
         case MVM_CALLSITE_ARG_STR:
@@ -207,7 +206,8 @@ MVMObject * MVM_capture_arg_pos_o(MVMThreadContext *tc, MVMObject *capture_obj,
     if (idx >= capture->body.callsite->num_pos)
         MVM_exception_throw_adhoc(tc, "Capture argument index (%u) out of range (0..^%u) for captureposarg", idx, capture->body.callsite->num_pos);
     if ((capture->body.callsite->arg_flags[idx] & MVM_CALLSITE_ARG_TYPE_MASK) != MVM_CALLSITE_ARG_OBJ)
-        MVM_exception_throw_adhoc(tc, "Capture argument is not an object argument for captureposarg");
+        MVM_exception_throw_adhoc(tc, "Capture argument is not an object argument for captureposarg. Got %d instead",
+            (capture->body.callsite->arg_flags[idx] & MVM_CALLSITE_ARG_TYPE_MASK));
     return capture->body.args[idx].o;
 }
 
@@ -236,11 +236,21 @@ MVMint64 MVM_capture_arg_pos_i(MVMThreadContext *tc, MVMObject *capture_obj, MVM
     MVMCapture *capture = validate_capture(tc, capture_obj);
     if (idx >= capture->body.callsite->num_pos)
         MVM_exception_throw_adhoc(tc, "Capture argument index (%u) out of range (0..^%u) for captureposarg_i", idx, capture->body.callsite->num_pos);
-    if ((capture->body.callsite->arg_flags[idx] & MVM_CALLSITE_ARG_TYPE_MASK) != MVM_CALLSITE_ARG_INT)
+    if (!(capture->body.callsite->arg_flags[idx] & (MVM_CALLSITE_ARG_INT | MVM_CALLSITE_ARG_UINT)))
         MVM_exception_throw_adhoc(tc, "Capture argument is not an integer argument for captureposarg_i");
     return capture->body.args[idx].i64;
 }
 
+/* Access a positional unsigned integer argument of an argument capture object. */
+MVMuint64 MVM_capture_arg_pos_u(MVMThreadContext *tc, MVMObject *capture_obj, MVMuint32 idx) {
+    MVMCapture *capture = validate_capture(tc, capture_obj);
+    if (idx >= capture->body.callsite->num_pos)
+        MVM_exception_throw_adhoc(tc, "Capture argument index (%u) out of range (0..^%u) for captureposarg_u", idx, capture->body.callsite->num_pos);
+    if (!(capture->body.callsite->arg_flags[idx] & MVM_CALLSITE_ARG_INT))
+        MVM_exception_throw_adhoc(tc, "Capture argument is not an integer argument for captureposarg_u");
+    return capture->body.args[idx].u64;
+}
+
 /* Access a positional number argument of an argument capture object. */
 MVMnum64 MVM_capture_arg_pos_n(MVMThreadContext *tc, MVMObject *capture_obj, MVMuint32 idx) {
     MVMCapture *capture = validate_capture(tc, capture_obj);
@@ -357,8 +367,7 @@ MVMObject * MVM_capture_drop_args(MVMThreadContext *tc, MVMObject *capture_obj,
     /* Form a new arguments buffer, dropping the specified argument. */
     MVMRegister *new_args;
     if (new_callsite->flag_count) {
-        new_args = MVM_fixed_size_alloc(tc, tc->instance->fsa,
-            new_callsite->flag_count * sizeof(MVMRegister));
+        new_args = MVM_malloc(new_callsite->flag_count * sizeof(MVMRegister));
         MVMuint32 from, to = 0;
         for (from = 0; from < capture->body.callsite->flag_count; from++) {
             if (from < idx || from >= idx + count) {
@@ -404,8 +413,7 @@ MVMObject * MVM_capture_insert_arg(MVMThreadContext *tc, MVMObject *capture_obj,
             idx, kind);
 
     /* Form a new arguments buffer, dropping the specified argument. */
-    MVMRegister *new_args = MVM_fixed_size_alloc(tc, tc->instance->fsa,
-            new_callsite->flag_count * sizeof(MVMRegister));
+    MVMRegister *new_args = MVM_malloc(new_callsite->flag_count * sizeof(MVMRegister));
     MVMuint32 from, to = 0;
     for (from = 0; from < capture->body.callsite->flag_count; from++) {
         if (from == idx) {
@@ -452,12 +460,14 @@ MVMObject * MVM_capture_replace_arg(MVMThreadContext *tc, MVMObject *capture_obj
      * The callsite MUST be created after we allocated as it may contain named
      * arguments, i.e. contain pointers to strings which wouldn't get marked. */
     MVMCallsite *callsite = capture->body.callsite;
-    MVMCallsite *new_callsite = MVM_callsite_replace_positional(tc, capture->body.callsite, idx, kind);
+    if ((callsite->arg_flags[idx] & MVM_CALLSITE_ARG_TYPE_MASK) != kind)
+        MVM_exception_throw_adhoc(tc, "Cannot replace capture argument with different kind %d -> %d", callsite->arg_flags[idx], kind);
+
+    MVMCallsite *new_callsite = MVM_callsite_replace_positional(tc, callsite, idx, kind);
     new_callsite->arg_flags[idx] = kind;
 
     /* Form a new arguments buffer, replacing the specified argument. */
-    MVMRegister *new_args = MVM_fixed_size_alloc(tc, tc->instance->fsa,
-            callsite->flag_count * sizeof(MVMRegister));
+    MVMRegister *new_args = MVM_malloc(callsite->flag_count * sizeof(MVMRegister));
     MVMuint32 from = 0;
     for (from = 0; from < capture->body.callsite->flag_count; from++) {
         new_args[from] = capture->body.args[from];
