@@ -448,13 +448,15 @@ MVMObject * MVM_context_lexical_lookup(MVMThreadContext *tc, MVMContext *ctx, MV
 
 /* Does a dynamic lexical lookup relative to the context's current location.
  * Evaluates to a VMNull if it's not found. */
-MVMObject * MVM_context_dynamic_lookup(MVMThreadContext *tc, MVMContext *ctx, MVMString *name) {
+void MVM_context_dynamic_lookup(MVMThreadContext *tc, MVMContext *ctx, MVMString *name, MVMRegister *r) {
     MVMSpeshFrameWalker fw;
     MVM_spesh_frame_walker_init(tc, &fw, ctx->body.context, 0);
-    if (apply_traversals(tc, &fw, ctx->body.traversals, ctx->body.num_traversals))
-        return MVM_frame_getdynlex_with_frame_walker(tc, &fw, name);
+    if (apply_traversals(tc, &fw, ctx->body.traversals, ctx->body.num_traversals)) {
+        MVM_frame_getdynlex_with_frame_walker(tc, &fw, name, r);
+        return;
+    }
     MVM_spesh_frame_walker_cleanup(tc, &fw);
-    return tc->instance->VMNull;
+    r->o = tc->instance->VMNull;
 }
 
 /* Does a caller lexical lookup relative to the context's current location.
