@@ -59,8 +59,7 @@ static MVMObject * allocate(MVMThreadContext *tc, MVMSTable *st) {
     MVMMultiDimArrayREPRData *repr_data = (MVMMultiDimArrayREPRData *)st->REPR_data;
     if (repr_data) {
         MVMObject *obj = MVM_gc_allocate_object(tc, st);
-        ((MVMMultiDimArray *)obj)->body.dimensions = MVM_fixed_size_alloc_zeroed(tc,
-            tc->instance->fsa, repr_data->num_dimensions * sizeof(MVMint64));
+        ((MVMMultiDimArray *)obj)->body.dimensions = MVM_calloc(repr_data->num_dimensions, sizeof(MVMint64));
         return obj;
     }
     else {
@@ -144,6 +143,41 @@ static void spec_to_repr_data(MVMThreadContext *tc, MVMMultiDimArrayREPRData *re
                 }
             }
             break;
+        case MVM_STORAGE_SPEC_BP_UINT64:
+            switch (spec->bits) {
+                case 64:
+                    repr_data->slot_type = MVM_ARRAY_U64;
+                    repr_data->elem_size = sizeof(MVMuint64);
+                    break;
+                case 32:
+                    repr_data->slot_type = MVM_ARRAY_U32;
+                    repr_data->elem_size = sizeof(MVMuint32);
+                    break;
+                case 16:
+                    repr_data->slot_type = MVM_ARRAY_U16;
+                    repr_data->elem_size = sizeof(MVMuint16);
+                    break;
+                case 8:
+                    repr_data->slot_type = MVM_ARRAY_U8;
+                    repr_data->elem_size = sizeof(MVMuint8);
+                    break;
+                case 4:
+                    repr_data->slot_type = MVM_ARRAY_U4;
+                    repr_data->elem_size = 0;
+                    break;
+                case 2:
+                    repr_data->slot_type = MVM_ARRAY_U2;
+                    repr_data->elem_size = 0;
+                    break;
+                case 1:
+                    repr_data->slot_type = MVM_ARRAY_U1;
+                    repr_data->elem_size = 0;
+                    break;
+                default:
+                    MVM_exception_throw_adhoc(tc,
+                        "MVMMultiDimArray: Unsupported uint size");
+            }
+            break;
         case MVM_STORAGE_SPEC_BP_NUM:
             switch (spec->bits) {
                 case 64:
@@ -214,8 +248,8 @@ static void copy_to(MVMThreadContext *tc, MVMSTable *st, void *src, MVMObject *d
     if (src_body->slots.any) {
         size_t dim_size  = repr_data->num_dimensions * sizeof(MVMint64);
         size_t data_size = flat_size(repr_data, src_body->dimensions);
-        dest_body->dimensions = MVM_fixed_size_alloc(tc, tc->instance->fsa, dim_size);
-        dest_body->slots.any  = MVM_fixed_size_alloc(tc, tc->instance->fsa, data_size);
+        dest_body->dimensions = MVM_malloc(dim_size);
+        dest_body->slots.any  = MVM_malloc(data_size);
         memcpy(dest_body->dimensions, src_body->dimensions, dim_size);
         memcpy(dest_body->slots.any, src_body->slots.any, data_size);
     }
@@ -248,14 +282,9 @@ static void gc_mark(MVMThreadContext *tc, MVMSTable *st, void *data, MVMGCWorkli
 /* Called by the VM in order to free memory associated with this object. */
 static void gc_free(MVMThreadContext *tc, MVMObject *obj) {
     MVMMultiDimArray *arr = (MVMMultiDimArray *)obj;
-    MVMMultiDimArrayREPRData *repr_data = (MVMMultiDimArrayREPRData *)STABLE(obj)->REPR_data;
     if (arr->body.slots.any)
-        MVM_fixed_size_free(tc, tc->instance->fsa,
-            flat_size(repr_data, arr->body.dimensions),
-            arr->body.slots.any);
-    MVM_fixed_size_free(tc, tc->instance->fsa,
-        repr_data->num_dimensions * sizeof(MVMint64),
-        arr->body.dimensions);
+        MVM_free(arr->body.slots.any);
+    MVM_free(arr->body.dimensions);
 }
 
 /* Marks the representation data in an STable.*/
@@ -351,8 +380,7 @@ static void deserialize(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, vo
         body->dimensions[i] = MVM_serialization_read_int(tc, reader);
 
     /* Allocate storage. */
-    body->slots.any = MVM_fixed_size_alloc_zeroed(tc, tc->instance->fsa,
-        flat_size(repr_data, body->dimensions));
+    body->slots.any = MVM_calloc(flat_elements(repr_data->num_dimensions, body->dimensions), repr_data->elem_size);
 
     /* Read in elements. */
     flat_elems = flat_elements(repr_data->num_dimensions, body->dimensions);
@@ -416,11 +444,7 @@ static void serialize_repr_data(MVMThreadContext *tc, MVMSTable *st, MVMSerializ
 static void deserialize_repr_data(MVMThreadContext *tc, MVMSTable *st, MVMSerializationReader *reader) {
     MVMint64 num_dims;
 
-    if (reader->root.version >= 19) {
-        num_dims = MVM_serialization_read_int(tc, reader);
-    } else {
-        num_dims = MVM_serialization_read_int64(tc, reader);
-    }
+    num_dims = MVM_serialization_read_int(tc, reader);
 
     if (num_dims > 0) {
         MVMMultiDimArrayREPRData *repr_data = (MVMMultiDimArrayREPRData *)MVM_malloc(sizeof(MVMMultiDimArrayREPRData));
@@ -524,28 +548,28 @@ static void at_pos_multidim(MVMThreadContext *tc, MVMSTable *st, MVMObject *root
                     MVM_exception_throw_adhoc(tc, "MultiDimArray: atpos expected num register");
                 break;
             case MVM_ARRAY_U64:
-                if (kind == MVM_reg_int64)
-                    value->i64 = (MVMint64)body->slots.u64[flat_index];
+                if (kind == MVM_reg_uint64)
+                    value->u64 = body->slots.u64[flat_index];
                 else
-                    MVM_exception_throw_adhoc(tc, "MultiDimArray: atpos expected int register");
+                    MVM_exception_throw_adhoc(tc, "MultiDimArray: atpos expected uint register");
                 break;
             case MVM_ARRAY_U32:
-                if (kind == MVM_reg_int64)
-                    value->i64 = (MVMint64)body->slots.u32[flat_index];
+                if (kind == MVM_reg_uint64)
+                    value->u64 = (MVMuint64)body->slots.u32[flat_index];
                 else
-                    MVM_exception_throw_adhoc(tc, "MultiDimArray: atpos expected int register");
+                    MVM_exception_throw_adhoc(tc, "MultiDimArray: atpos expected uint register");
                 break;
             case MVM_ARRAY_U16:
-                if (kind == MVM_reg_int64)
-                    value->i64 = (MVMint64)body->slots.u16[flat_index];
+                if (kind == MVM_reg_uint64)
+                    value->u64 = (MVMuint64)body->slots.u16[flat_index];
                 else
-                    MVM_exception_throw_adhoc(tc, "MultiDimArray: atpos expected int register");
+                    MVM_exception_throw_adhoc(tc, "MultiDimArray: atpos expected uint register");
                 break;
             case MVM_ARRAY_U8:
-                if (kind == MVM_reg_int64)
-                    value->i64 = (MVMint64)body->slots.u8[flat_index];
+                if (kind == MVM_reg_uint64)
+                    value->u64 = (MVMuint64)body->slots.u8[flat_index];
                 else
-                    MVM_exception_throw_adhoc(tc, "MultiDimArray: atpos expected int register");
+                    MVM_exception_throw_adhoc(tc, "MultiDimArray: atpos expected uint register");
                 break;
             default:
                 MVM_exception_throw_adhoc(tc, "MultiDimArray: Unhandled slot type");
@@ -617,28 +641,28 @@ static void bind_pos_multidim(MVMThreadContext *tc, MVMSTable *st, MVMObject *ro
                     MVM_exception_throw_adhoc(tc, "MultiDimArray: bindpos expected num register");
                 break;
             case MVM_ARRAY_U64:
-                if (kind == MVM_reg_int64)
-                    body->slots.u64[flat_index] = value.i64;
+                if (kind == MVM_reg_uint64)
+                    body->slots.u64[flat_index] = value.u64;
                 else
-                    MVM_exception_throw_adhoc(tc, "MultiDimArray: bindpos expected int register");
+                    MVM_exception_throw_adhoc(tc, "MultiDimArray: bindpos expected uint register");
                 break;
             case MVM_ARRAY_U32:
-                if (kind == MVM_reg_int64)
-                    body->slots.u32[flat_index] = (MVMuint32)value.i64;
+                if (kind == MVM_reg_uint64)
+                    body->slots.u32[flat_index] = (MVMuint32)value.u64;
                 else
-                    MVM_exception_throw_adhoc(tc, "MultiDimArray: bindpos expected int register");
+                    MVM_exception_throw_adhoc(tc, "MultiDimArray: bindpos expected uint register");
                 break;
             case MVM_ARRAY_U16:
-                if (kind == MVM_reg_int64)
-                    body->slots.u16[flat_index] = (MVMuint16)value.i64;
+                if (kind == MVM_reg_uint64)
+                    body->slots.u16[flat_index] = (MVMuint16)value.u64;
                 else
-                    MVM_exception_throw_adhoc(tc, "MultiDimArray: bindpos expected int register");
+                    MVM_exception_throw_adhoc(tc, "MultiDimArray: bindpos expected uint register");
                 break;
             case MVM_ARRAY_U8:
-                if (kind == MVM_reg_int64)
-                    body->slots.u8[flat_index] = (MVMuint8)value.i64;
+                if (kind == MVM_reg_uint64)
+                    body->slots.u8[flat_index] = (MVMuint8)value.u64;
                 else
-                    MVM_exception_throw_adhoc(tc, "MultiDimArray: bindpos expected int register");
+                    MVM_exception_throw_adhoc(tc, "MultiDimArray: bindpos expected uint register");
                 break;
             default:
                 MVM_exception_throw_adhoc(tc, "MultiDimArray: Unhandled slot type");
@@ -673,8 +697,7 @@ static void set_dimensions(MVMThreadContext *tc, MVMSTable *st, MVMObject *root,
          * real world use case, but we should ensure the VM is memory safe).
          */
         MVMMultiDimArrayBody *body = (MVMMultiDimArrayBody *)data;
-        size_t size = flat_size(repr_data, dimensions);
-        void *storage = MVM_fixed_size_alloc_zeroed(tc, tc->instance->fsa, size);
+        void *storage = MVM_calloc(flat_elements(repr_data->num_dimensions, dimensions), repr_data->elem_size);
         if (MVM_trycas(&(body->slots.any), NULL, storage)) {
             /* Now memory is in place, safe to un-zero dimensions. */
             memcpy(body->dimensions, dimensions, num_dimensions * sizeof(MVMint64));
@@ -756,9 +779,9 @@ static MVMStorageSpec get_elem_storage_spec(MVMThreadContext *tc, MVMSTable *st)
     return spec;
 }
 
-AO_t * pos_as_atomic_multidim(MVMThreadContext *tc, MVMSTable *st,
-                              MVMObject *root, void *data,
-                              MVMint64 num_indices, MVMint64 *indices) {
+static AO_t * pos_as_atomic_multidim(MVMThreadContext *tc, MVMSTable *st,
+                                     MVMObject *root, void *data,
+                                     MVMint64 num_indices, MVMint64 *indices) {
     MVMMultiDimArrayREPRData *repr_data = (MVMMultiDimArrayREPRData *)st->REPR_data;
     if (num_indices == repr_data->num_dimensions) {
         MVMMultiDimArrayBody *body = (MVMMultiDimArrayBody *)data;
@@ -814,7 +837,9 @@ static const MVMREPROps MultiDimArray_this_repr = {
         set_dimensions,
         get_elem_storage_spec,
         pos_as_atomic,
-        pos_as_atomic_multidim
+        pos_as_atomic_multidim,
+        MVM_REPR_DEFAULT_POS_WRITE_BUF,
+        MVM_REPR_DEFAULT_POS_READ_BUF,
     },
     MVM_REPR_DEFAULT_ASS_FUNCS,
     elems,
