@@ -15,8 +15,7 @@ MVM_STATIC_INLINE void mark_named_used(MVMArgProcContext *ctx, MVMuint32 idx) {
 
 /* An identity map is just an array { 0, 1, 2, ... }. */
 static MVMuint16 * create_identity_map(MVMThreadContext *tc, MVMuint32 size) {
-    MVMuint16 *map = MVM_fixed_size_alloc(tc, tc->instance->fsa,
-            size * sizeof(MVMuint16));
+    MVMuint16 *map = MVM_malloc(size * sizeof(MVMuint16));
     MVMuint32 i;
     for (i = 0; i < size; i++)
         map[i] = i;
@@ -34,13 +33,9 @@ void MVM_args_setup_identity_map(MVMThreadContext *tc) {
 
 /* Free memory associated with the identity map(s). */
 void MVM_args_destroy_identity_map(MVMThreadContext *tc) {
-    MVM_fixed_size_free(tc, tc->instance->fsa,
-            tc->instance->identity_arg_map_alloc * sizeof(MVMuint16),
-            tc->instance->identity_arg_map);
+    MVM_free(tc->instance->identity_arg_map);
     if (tc->instance->identity_arg_map != tc->instance->small_identity_arg_map)
-        MVM_fixed_size_free(tc, tc->instance->fsa,
-                MVM_ARGS_SMALL_IDENTITY_MAP_SIZE * sizeof(MVMuint16),
-                tc->instance->small_identity_arg_map);
+        MVM_free(tc->instance->small_identity_arg_map);
 }
 
 /* Perform flattening of arguments as provided, and return the resulting
@@ -691,6 +686,9 @@ void MVM_args_set_result_obj(MVMThreadContext *tc, MVMObject *result, MVMint32 f
             case MVM_RETURN_INT:
                 target->return_value->i64 = MVM_repr_get_int(tc, decont_result(tc, result));
                 break;
+            case MVM_RETURN_UINT:
+                target->return_value->u64 = MVM_repr_get_uint(tc, decont_result(tc, result));
+                break;
             case MVM_RETURN_NUM:
                 target->return_value->n64 = MVM_repr_get_num(tc, decont_result(tc, result));
                 break;
@@ -717,6 +715,9 @@ void MVM_args_set_dispatch_result_obj(MVMThreadContext *tc, MVMFrame *target, MV
         case MVM_RETURN_INT:
             target->return_value->i64 = MVM_repr_get_int(tc, decont_result(tc, result));
             break;
+        case MVM_RETURN_UINT:
+            target->return_value->u64 = MVM_repr_get_uint(tc, decont_result(tc, result));
+            break;
         case MVM_RETURN_NUM:
             target->return_value->n64 = MVM_repr_get_num(tc, decont_result(tc, result));
             break;
@@ -750,6 +751,9 @@ void MVM_args_set_result_int(MVMThreadContext *tc, MVMint64 result, MVMint32 fra
             case MVM_RETURN_INT:
                 target->return_value->i64 = result;
                 break;
+            case MVM_RETURN_UINT:
+                target->return_value->u64 = result;
+                break;
             case MVM_RETURN_NUM:
                 target->return_value->n64 = (MVMnum64)result;
                 break;
@@ -769,6 +773,50 @@ void MVM_args_set_result_int(MVMThreadContext *tc, MVMint64 result, MVMint32 fra
     }
 }
 
+void MVM_args_set_result_uint(MVMThreadContext *tc, MVMuint64 result, MVMint32 frameless) {
+    MVMFrame *target;
+    if (frameless) {
+        target = tc->cur_frame;
+    }
+    else {
+        if (MVM_spesh_log_is_caller_logging(tc))
+            MVM_spesh_log_return_type(tc, NULL);
+        else if (MVM_spesh_log_is_logging(tc))
+            MVM_spesh_log_return_to_unlogged(tc);
+        target = tc->cur_frame->caller;
+    }
+    if (target) {
+        switch (target->return_type) {
+            case MVM_RETURN_VOID:
+                if (tc->cur_frame->static_info->body.has_exit_handler)
+                    save_for_exit_handler(tc,
+                        MVM_repr_box_int(tc, MVM_hll_current(tc)->int_box_type, result));
+                break;
+            case MVM_RETURN_INT:
+                target->return_value->i64 = result;
+                break;
+            case MVM_RETURN_UINT:
+                target->return_value->u64 = result;
+                break;
+            case MVM_RETURN_NUM:
+                target->return_value->n64 = (MVMnum64)result;
+                break;
+            case MVM_RETURN_OBJ: {
+                /* dereference target first to avoid GC issue */
+                MVMRegister *return_value = (frameless ? tc->cur_frame : tc->cur_frame->caller)->return_value;
+                autobox_int(tc, target, result, return_value->o);
+                break;
+            }
+            case MVM_RETURN_ALLOMORPH:
+                target->return_type = MVM_RETURN_UINT;
+                target->return_value->u64 = result;
+                break;
+            default:
+                MVM_exception_throw_adhoc(tc, "Result return coercion from uint NYI; expects type %u", target->return_type);
+        }
+    }
+}
+
 void MVM_args_set_dispatch_result_int(MVMThreadContext *tc, MVMFrame *target, MVMint64 result) {
     switch (target->return_type) {
         case MVM_RETURN_VOID:
@@ -776,6 +824,32 @@ void MVM_args_set_dispatch_result_int(MVMThreadContext *tc, MVMFrame *target, MV
         case MVM_RETURN_INT:
             target->return_value->i64 = result;
             break;
+        case MVM_RETURN_UINT:
+            target->return_value->u64 = result;
+            break;
+        case MVM_RETURN_NUM:
+            target->return_value->n64 = (MVMnum64)result;
+            break;
+        case MVM_RETURN_OBJ: {
+            MVMRegister *return_value = target->return_value; /* dereference target first to avoid GC issue */
+            autobox_int(tc, target, result, return_value->o);
+            break;
+        }
+        default:
+            MVM_exception_throw_adhoc(tc, "Result return coercion from int NYI; expects type %u", target->return_type);
+    }
+}
+
+void MVM_args_set_dispatch_result_uint(MVMThreadContext *tc, MVMFrame *target, MVMuint64 result) {
+    switch (target->return_type) {
+        case MVM_RETURN_VOID:
+            break;
+        case MVM_RETURN_INT:
+            target->return_value->i64 = result;
+            break;
+        case MVM_RETURN_UINT:
+            target->return_value->u64 = result;
+            break;
         case MVM_RETURN_NUM:
             target->return_value->n64 = (MVMnum64)result;
             break;
@@ -814,6 +888,9 @@ void MVM_args_set_result_num(MVMThreadContext *tc, MVMnum64 result, MVMint32 fra
             case MVM_RETURN_INT:
                 target->return_value->i64 = (MVMint64)result;
                 break;
+            case MVM_RETURN_UINT:
+                target->return_value->u64 = (MVMuint64)result;
+                break;
             case MVM_RETURN_OBJ: {
                 /* dereference target first to avoid GC issue */
                 MVMRegister *return_value = (frameless ? tc->cur_frame : tc->cur_frame->caller)->return_value;
@@ -840,6 +917,9 @@ void MVM_args_set_dispatch_result_num(MVMThreadContext *tc, MVMFrame *target, MV
         case MVM_RETURN_INT:
             target->return_value->i64 = (MVMint64)result;
             break;
+        case MVM_RETURN_UINT:
+            target->return_value->u64 = (MVMuint64)result;
+            break;
         case MVM_RETURN_OBJ: {
             MVMRegister *return_value = target->return_value; /* dereference target first to avoid GC issue */
             autobox(tc, target, result, num_box_type, 0, set_num, return_value->o);
