@@ -28,6 +28,7 @@ MVMHLLConfig *MVM_hll_get_config_for(MVMThreadContext *tc, MVMString *name) {
         entry->hash_iterator_type = tc->instance->boot_types.BOOTIter;
         entry->max_inline_size = MVM_SPESH_DEFAULT_MAX_INLINE_SIZE;
         MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->int_box_type, "HLL int_box_type");
+        MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->uint_box_type, "HLL uint_box_type");
         MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->num_box_type, "HLL num_box_type");
         MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->str_box_type, "HLL str_box_type");
         MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->slurpy_array_type, "HLL slurpy_array_type");
@@ -41,15 +42,19 @@ MVMHLLConfig *MVM_hll_get_config_for(MVMThreadContext *tc, MVMString *name) {
         MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->method_not_found_error, "HLL method_not_found_error");
         MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->lexical_handler_not_found_error, "HLL lexical_handler_not_found_error");
         MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->int_lex_ref, "HLL int_lex_ref");
+        MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->uint_lex_ref, "HLL uint_lex_ref");
         MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->num_lex_ref, "HLL num_lex_ref");
         MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->str_lex_ref, "HLL str_lex_ref");
         MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->int_attr_ref, "HLL int_attr_ref");
+        MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->uint_attr_ref, "HLL uint_attr_ref");
         MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->num_attr_ref, "HLL num_attr_ref");
         MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->str_attr_ref, "HLL str_attr_ref");
         MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->int_pos_ref, "HLL int_pos_ref");
+        MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->uint_pos_ref, "HLL uint_pos_ref");
         MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->num_pos_ref, "HLL num_pos_ref");
         MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->str_pos_ref, "HLL str_pos_ref");
         MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->int_multidim_ref, "HLL int_multidim_ref");
+        MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->uint_multidim_ref, "HLL uint_multidim_ref");
         MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->num_multidim_ref, "HLL num_multidim_ref");
         MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->str_multidim_ref, "HLL str_multidim_ref");
         MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->call_dispatcher, "HLL call dispatcher name");
@@ -94,7 +99,7 @@ MVMHLLConfig *MVM_hll_get_config_for(MVMThreadContext *tc, MVMString *name) {
     MVMObject *val = MVM_repr_at_key_o((tc), (hash), key); \
     if (!MVM_is_null(tc, val)) (config)->member = MVM_repr_get_str(tc, val); \
 } while (0)
-void set_max_inline_size(MVMThreadContext *tc, MVMObject *config_hash, MVMHLLConfig *config) {
+static void set_max_inline_size(MVMThreadContext *tc, MVMObject *config_hash, MVMHLLConfig *config) {
     MVMROOT(tc, config_hash, {
         MVMString *key = MVM_string_ascii_decode_nt(tc, tc->instance->VMString, "max_inline_size");
         MVMObject *size = MVM_repr_at_key_o(tc, config_hash, key);
@@ -102,6 +107,9 @@ void set_max_inline_size(MVMThreadContext *tc, MVMObject *config_hash, MVMHLLCon
             config->max_inline_size = MVM_repr_get_int(tc, size);
     });
 }
+void MVM_hll_set_config_key(MVMThreadContext *tc, MVMHLLConfig *hll, MVMString *key, MVMObject *value) {
+    hll->uint_box_type = value;
+}
 MVMObject * MVM_hll_set_config(MVMThreadContext *tc, MVMString *name, MVMObject *config_hash) {
     MVMHLLConfig *config;
 
@@ -115,6 +123,7 @@ MVMObject * MVM_hll_set_config(MVMThreadContext *tc, MVMString *name, MVMObject
     /* MVM_string_utf8_decode() can potentially allocate, and hence gc. */
     MVMROOT(tc, config_hash, {
             check_config_key(tc, config_hash, "int_box", int_box_type, config);
+            check_config_key(tc, config_hash, "uint_box", uint_box_type, config);
             check_config_key(tc, config_hash, "num_box", num_box_type, config);
             check_config_key(tc, config_hash, "str_box", str_box_type, config);
             check_config_key(tc, config_hash, "slurpy_array", slurpy_array_type, config);
@@ -131,24 +140,32 @@ MVMObject * MVM_hll_set_config(MVMThreadContext *tc, MVMString *name, MVMObject
             check_config_key(tc, config_hash, "false_value", false_value, config);
             check_config_key_reftype(tc, config_hash, "int_lex_ref", int_lex_ref,
                 config, MVM_STORAGE_SPEC_BP_INT, MVM_NATIVEREF_LEX);
+            check_config_key_reftype(tc, config_hash, "uint_lex_ref", uint_lex_ref,
+                config, MVM_STORAGE_SPEC_BP_UINT64, MVM_NATIVEREF_LEX);
             check_config_key_reftype(tc, config_hash, "num_lex_ref", num_lex_ref,
                 config, MVM_STORAGE_SPEC_BP_NUM, MVM_NATIVEREF_LEX);
             check_config_key_reftype(tc, config_hash, "str_lex_ref", str_lex_ref,
                 config, MVM_STORAGE_SPEC_BP_STR, MVM_NATIVEREF_LEX);
             check_config_key_reftype(tc, config_hash, "int_attr_ref", int_attr_ref,
                 config, MVM_STORAGE_SPEC_BP_INT, MVM_NATIVEREF_ATTRIBUTE);
+            check_config_key_reftype(tc, config_hash, "uint_attr_ref", uint_attr_ref,
+                config, MVM_STORAGE_SPEC_BP_UINT64, MVM_NATIVEREF_ATTRIBUTE);
             check_config_key_reftype(tc, config_hash, "num_attr_ref", num_attr_ref,
                 config, MVM_STORAGE_SPEC_BP_NUM, MVM_NATIVEREF_ATTRIBUTE);
             check_config_key_reftype(tc, config_hash, "str_attr_ref", str_attr_ref,
                 config, MVM_STORAGE_SPEC_BP_STR, MVM_NATIVEREF_ATTRIBUTE);
             check_config_key_reftype(tc, config_hash, "int_pos_ref", int_pos_ref,
                 config, MVM_STORAGE_SPEC_BP_INT, MVM_NATIVEREF_POSITIONAL);
+            check_config_key_reftype(tc, config_hash, "uint_pos_ref", uint_pos_ref,
+                config, MVM_STORAGE_SPEC_BP_UINT64, MVM_NATIVEREF_POSITIONAL);
             check_config_key_reftype(tc, config_hash, "num_pos_ref", num_pos_ref,
                 config, MVM_STORAGE_SPEC_BP_NUM, MVM_NATIVEREF_POSITIONAL);
             check_config_key_reftype(tc, config_hash, "str_pos_ref", str_pos_ref,
                 config, MVM_STORAGE_SPEC_BP_STR, MVM_NATIVEREF_POSITIONAL);
             check_config_key_reftype(tc, config_hash, "int_multidim_ref", int_multidim_ref,
                 config, MVM_STORAGE_SPEC_BP_INT, MVM_NATIVEREF_MULTIDIM);
+            check_config_key_reftype(tc, config_hash, "uint_multidim_ref", uint_multidim_ref,
+                config, MVM_STORAGE_SPEC_BP_UINT64, MVM_NATIVEREF_MULTIDIM);
             check_config_key_reftype(tc, config_hash, "num_multidim_ref", num_multidim_ref,
                 config, MVM_STORAGE_SPEC_BP_NUM, MVM_NATIVEREF_MULTIDIM);
             check_config_key_reftype(tc, config_hash, "str_multidim_ref", str_multidim_ref,
