@@ -1086,18 +1086,9 @@ void MVM_nativecall_dispatch(MVMThreadContext *tc, MVMObject *res_type,
                 case MVM_NATIVECALL_ARG_UTF8STR:
                 case MVM_NATIVECALL_ARG_UTF16STR:
                     {
-                        char *str;
-                        switch (arg_types[i] & MVM_NATIVECALL_ARG_TYPE_MASK) {
-                            case MVM_NATIVECALL_ARG_ASCIISTR:
-                                str = MVM_string_ascii_encode_any(tc, value);
-                                break;
-                            case MVM_NATIVECALL_ARG_UTF16STR:
-                                str = MVM_string_utf16_encode(tc, value, 0);
-                                break;
-                            default:
-                                str = MVM_string_utf8_encode_C_string(tc, value);
-                        }
-                        if (arg_types[i] & MVM_NATIVECALL_ARG_FREE_STR_MASK) {
+                        MVMint16 free = 0;
+                        char *str = MVM_nativecall_encode_string(tc, value, arg_types[i], &free, i, REPR(value));
+                        if (free) {
                             if (!free_strs)
                                 free_strs = (char**)alloca(num_args * sizeof(char *));
                             free_strs[num_strs] = str;
@@ -1220,6 +1211,10 @@ void MVM_nativecall_dispatch(MVMThreadContext *tc, MVMObject *res_type,
                         update_rws(tc, free_rws, num_args, arg_types, args, interval_id);
                         MVM_args_set_dispatch_result_int(tc, tc->cur_frame, (MVMuint64)native_result);
                     }
+                    else if (tc->cur_frame->return_type == MVM_RETURN_UINT) {
+                        update_rws(tc, free_rws, num_args, arg_types, args, interval_id);
+                        MVM_args_set_dispatch_result_uint(tc, tc->cur_frame, (MVMuint64)native_result);
+                    }
                     else {
                         result = MVM_nativecall_make_cpointer(tc, res_type, native_result);
                         update_rws(tc, free_rws, num_args, arg_types, args, interval_id);
@@ -1257,35 +1252,35 @@ void MVM_nativecall_dispatch(MVMThreadContext *tc, MVMObject *res_type,
                     MVMuint64 native_result = (DCuchar)dcCallChar(vm, entry_point);
                     MVM_gc_mark_thread_unblocked(tc);
                     update_rws(tc, free_rws, num_args, arg_types, args, interval_id);
-                    MVM_args_set_dispatch_result_int(tc, tc->cur_frame, native_result);
+                    MVM_args_set_dispatch_result_uint(tc, tc->cur_frame, native_result);
                     break;
                 }
                 case MVM_NATIVECALL_ARG_USHORT: {
                     MVMuint64 native_result = (DCushort)dcCallShort(vm, entry_point);
                     MVM_gc_mark_thread_unblocked(tc);
                     update_rws(tc, free_rws, num_args, arg_types, args, interval_id);
-                    MVM_args_set_dispatch_result_int(tc, tc->cur_frame, native_result);
+                    MVM_args_set_dispatch_result_uint(tc, tc->cur_frame, native_result);
                     break;
                 }
                 case MVM_NATIVECALL_ARG_UINT: {
                     MVMuint64 native_result = (DCuint)dcCallInt(vm, entry_point);
                     MVM_gc_mark_thread_unblocked(tc);
                     update_rws(tc, free_rws, num_args, arg_types, args, interval_id);
-                    MVM_args_set_dispatch_result_int(tc, tc->cur_frame, native_result);
+                    MVM_args_set_dispatch_result_uint(tc, tc->cur_frame, native_result);
                     break;
                 }
                 case MVM_NATIVECALL_ARG_ULONG: {
                     MVMuint64 native_result = (DCulong)dcCallLong(vm, entry_point);
                     MVM_gc_mark_thread_unblocked(tc);
                     update_rws(tc, free_rws, num_args, arg_types, args, interval_id);
-                    MVM_args_set_dispatch_result_int(tc, tc->cur_frame, native_result);
+                    MVM_args_set_dispatch_result_uint(tc, tc->cur_frame, native_result);
                     break;
                 }
                 case MVM_NATIVECALL_ARG_ULONGLONG: {
                     MVMuint64 native_result = (DCulonglong)dcCallLongLong(vm, entry_point);
                     MVM_gc_mark_thread_unblocked(tc);
                     update_rws(tc, free_rws, num_args, arg_types, args, interval_id);
-                    MVM_args_set_dispatch_result_int(tc, tc->cur_frame, native_result);
+                    MVM_args_set_dispatch_result_uint(tc, tc->cur_frame, native_result);
                     break;
                 }
                 default:
@@ -1301,7 +1296,11 @@ void MVM_nativecall_dispatch(MVMThreadContext *tc, MVMObject *res_type,
     /* Free any memory that we need to. */
     if (free_strs)
         for (i = 0; i < num_strs; i++)
+#ifdef MVM_USE_MIMALLOC
+            free(free_strs[i]);
+#else
             MVM_free(free_strs[i]);
+#endif
 
     /* Finally, free call VM. */
     dcFree(vm);
