@@ -58,10 +58,12 @@ MVMCallsite * MVM_spesh_disp_callsite_for_dispatch_op(MVMuint16 opcode, MVMuint8
         case MVM_OP_sp_dispatch_v:
             return cu->body.callsites[GET_UI16(args, 4)];
         case MVM_OP_dispatch_i:
+        case MVM_OP_dispatch_u:
         case MVM_OP_dispatch_n:
         case MVM_OP_dispatch_s:
         case MVM_OP_dispatch_o:
         case MVM_OP_sp_dispatch_i:
+        case MVM_OP_sp_dispatch_u:
         case MVM_OP_sp_dispatch_n:
         case MVM_OP_sp_dispatch_s:
         case MVM_OP_sp_dispatch_o:
@@ -71,14 +73,17 @@ MVMCallsite * MVM_spesh_disp_callsite_for_dispatch_op(MVMuint16 opcode, MVMuint8
         case MVM_OP_sp_runnativecall_v:
             return (MVMCallsite *)GET_UI64(args, 2);
         case MVM_OP_sp_runbytecode_i:
+        case MVM_OP_sp_runbytecode_u:
         case MVM_OP_sp_runbytecode_n:
         case MVM_OP_sp_runbytecode_s:
         case MVM_OP_sp_runbytecode_o:
         case MVM_OP_sp_runcfunc_i:
+        case MVM_OP_sp_runcfunc_u:
         case MVM_OP_sp_runcfunc_n:
         case MVM_OP_sp_runcfunc_s:
         case MVM_OP_sp_runcfunc_o:
         case MVM_OP_sp_runnativecall_i:
+        case MVM_OP_sp_runnativecall_u:
         case MVM_OP_sp_runnativecall_n:
         case MVM_OP_sp_runnativecall_s:
         case MVM_OP_sp_runnativecall_o:
@@ -107,6 +112,7 @@ static void rewrite_to_sp_dispatch(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSp
     switch (ins->info->opcode) {
         case MVM_OP_dispatch_v: new_ins_base_info = MVM_op_get_op(MVM_OP_sp_dispatch_v); break;
         case MVM_OP_dispatch_i: new_ins_base_info = MVM_op_get_op(MVM_OP_sp_dispatch_i); break;
+        case MVM_OP_dispatch_u: new_ins_base_info = MVM_op_get_op(MVM_OP_sp_dispatch_u); break;
         case MVM_OP_dispatch_n: new_ins_base_info = MVM_op_get_op(MVM_OP_sp_dispatch_n); break;
         case MVM_OP_dispatch_o: new_ins_base_info = MVM_op_get_op(MVM_OP_sp_dispatch_o); break;
         case MVM_OP_dispatch_s: new_ins_base_info = MVM_op_get_op(MVM_OP_sp_dispatch_s); break;
@@ -141,6 +147,7 @@ static MVMuint32 find_disp_op_first_real_arg(MVMThreadContext *tc, MVMSpeshIns *
     if (ins->info->opcode == MVM_OP_dispatch_v)
         return 2;
     assert(ins->info->opcode == MVM_OP_dispatch_i ||
+            ins->info->opcode == MVM_OP_dispatch_u ||
             ins->info->opcode == MVM_OP_dispatch_n ||
             ins->info->opcode == MVM_OP_dispatch_s ||
             ins->info->opcode == MVM_OP_dispatch_o);
@@ -625,6 +632,7 @@ static int translate_dispatch_program(MVMThreadContext *tc, MVMSpeshGraph *g,
 
     /* Visit the ops of the dispatch program and translate them. */
     MVMSpeshIns *insert_after = ins;
+    MVMSpeshIns *orig_next = ins->next;
     MVMCallsite *callsite = NULL;
     MVMint32 skip_args = -1;
     for (i = 0; i < dp->num_ops; i++) {
@@ -891,15 +899,21 @@ static int translate_dispatch_program(MVMThreadContext *tc, MVMSpeshGraph *g,
                 MVMSpeshOperand constint = { .lit_i64 = dp->constants[op->load.idx].i64 };
                 emit_bi_op(tc, g, bb, &insert_after, MVM_OP_const_i64,
                     temporaries[op->load.temp], constint);
+                MVMSpeshFacts *f = MVM_spesh_get_facts(tc, g, temp);
+                f->flags |= MVM_SPESH_FACT_KNOWN_VALUE;
+                f->value.i = constint.lit_i64;
                 break;
             }
             case MVMDispOpcodeLoadConstantNum: {
                 MVMSpeshOperand temp = MVM_spesh_manipulate_get_temp_reg(tc, g, MVM_reg_num64);
                 temporaries[op->load.temp] = temp;
                 MVM_VECTOR_PUSH(allocated_temps, temp);
-                MVMSpeshOperand constnum = { .lit_i64 = dp->constants[op->load.idx].n64 };
+                MVMSpeshOperand constnum = { .lit_n64 = dp->constants[op->load.idx].n64 };
                 emit_bi_op(tc, g, bb, &insert_after, MVM_OP_const_n64,
                     temporaries[op->load.temp], constnum);
+                MVMSpeshFacts *f = MVM_spesh_get_facts(tc, g, temp);
+                f->flags |= MVM_SPESH_FACT_KNOWN_VALUE;
+                f->value.n = constnum.lit_n64;
                 break;
             }
             case MVMDispOpcodeLoadAttributeObj: {
@@ -994,6 +1008,10 @@ static int translate_dispatch_program(MVMThreadContext *tc, MVMSpeshGraph *g,
                         emit_bi_op(tc, g, bb, &insert_after, MVM_OP_unbox_i, ins->operands[0],
                             temporaries[op->res_value.temp]);
                         break;
+                    case MVM_OP_dispatch_u:
+                        emit_bi_op(tc, g, bb, &insert_after, MVM_OP_unbox_u, ins->operands[0],
+                            temporaries[op->res_value.temp]);
+                        break;
                     case MVM_OP_dispatch_n:
                         emit_bi_op(tc, g, bb, &insert_after, MVM_OP_unbox_n, ins->operands[0],
                             temporaries[op->res_value.temp]);
@@ -1025,6 +1043,10 @@ static int translate_dispatch_program(MVMThreadContext *tc, MVMSpeshGraph *g,
                         emit_bi_op(tc, g, bb, &insert_after, MVM_OP_coerce_si, ins->operands[0],
                             temporaries[op->res_value.temp]);
                         break;
+                    case MVM_OP_dispatch_u:
+                        emit_bi_op(tc, g, bb, &insert_after, MVM_OP_coerce_su, ins->operands[0],
+                            temporaries[op->res_value.temp]);
+                        break;
                     case MVM_OP_dispatch_n:
                         emit_bi_op(tc, g, bb, &insert_after, MVM_OP_coerce_sn, ins->operands[0],
                             temporaries[op->res_value.temp]);
@@ -1056,6 +1078,10 @@ static int translate_dispatch_program(MVMThreadContext *tc, MVMSpeshGraph *g,
                         emit_bi_op(tc, g, bb, &insert_after, MVM_OP_set, ins->operands[0],
                             temporaries[op->res_value.temp]);
                         break;
+                    case MVM_OP_dispatch_u:
+                        emit_bi_op(tc, g, bb, &insert_after, MVM_OP_set, ins->operands[0],
+                            temporaries[op->res_value.temp]);
+                        break;
                     case MVM_OP_dispatch_n:
                         emit_bi_op(tc, g, bb, &insert_after, MVM_OP_coerce_in, ins->operands[0],
                             temporaries[op->res_value.temp]);
@@ -1087,6 +1113,10 @@ static int translate_dispatch_program(MVMThreadContext *tc, MVMSpeshGraph *g,
                         emit_bi_op(tc, g, bb, &insert_after, MVM_OP_coerce_ni, ins->operands[0],
                             temporaries[op->res_value.temp]);
                         break;
+                    case MVM_OP_dispatch_u:
+                        emit_bi_op(tc, g, bb, &insert_after, MVM_OP_coerce_nu, ins->operands[0],
+                            temporaries[op->res_value.temp]);
+                        break;
                     case MVM_OP_dispatch_n:
                         emit_bi_op(tc, g, bb, &insert_after, MVM_OP_set, ins->operands[0],
                             temporaries[op->res_value.temp]);
@@ -1128,6 +1158,9 @@ static int translate_dispatch_program(MVMThreadContext *tc, MVMSpeshGraph *g,
                     case MVM_OP_dispatch_i:
                         base_op = MVM_op_get_op(MVM_OP_sp_runnativecall_i);
                         break;
+                    case MVM_OP_dispatch_u:
+                        base_op = MVM_op_get_op(MVM_OP_sp_runnativecall_u);
+                        break;
                     case MVM_OP_dispatch_n:
                         base_op = MVM_op_get_op(MVM_OP_sp_runnativecall_n);
                         break;
@@ -1204,6 +1237,8 @@ static int translate_dispatch_program(MVMThreadContext *tc, MVMSpeshGraph *g,
                     for (j = 1; j < callsite->flag_count; j++) { /* first arg is return type */
                         if ((arg_types[j - 1] & MVM_NATIVECALL_ARG_RW_MASK) == MVM_NATIVECALL_ARG_RW) {
                             switch (arg_types[j - 1] & MVM_NATIVECALL_ARG_TYPE_MASK) {
+                                case MVM_NATIVECALL_ARG_CHAR:
+                                case MVM_NATIVECALL_ARG_SHORT:
                                 case MVM_NATIVECALL_ARG_INT:
                                 case MVM_NATIVECALL_ARG_LONG:
                                 case MVM_NATIVECALL_ARG_LONGLONG: {
@@ -1212,8 +1247,27 @@ static int translate_dispatch_program(MVMThreadContext *tc, MVMSpeshGraph *g,
                                     rw_operands[j] = MVM_spesh_manipulate_get_temp_reg(tc, g, MVM_reg_int64);
                                     emit_bi_op(tc, g, bb, &insert_after, MVM_OP_decont_i, rw_operands[j], var);
 
-                                    callsite = MVM_callsite_drop_positional(tc, callsite, j);
-                                    callsite = MVM_callsite_insert_positional(tc, callsite, j, MVM_CALLSITE_ARG_INT);
+                                    callsite = MVM_callsite_replace_positional(tc, callsite, j, MVM_CALLSITE_ARG_INT);
+                                    MVM_callsite_intern(tc, &callsite, 1, 0);
+
+                                    if (has_return_value && var.reg.orig == rb_ins->operands[0].reg.orig) {
+                                        rw_dummy = MVM_spesh_manipulate_get_temp_reg(tc, g, MVM_reg_obj);
+                                        emit_bi_op(tc, g, bb, &insert_after, MVM_OP_set, rw_dummy, var);
+                                        has_rw_dummy = 1;
+                                    }
+                                    break;
+                                }
+                                case MVM_NATIVECALL_ARG_UCHAR:
+                                case MVM_NATIVECALL_ARG_USHORT:
+                                case MVM_NATIVECALL_ARG_UINT:
+                                case MVM_NATIVECALL_ARG_ULONG:
+                                case MVM_NATIVECALL_ARG_ULONGLONG: {
+                                    MVMSpeshOperand var = skip_args >= 0 ? args[skip_args + j] : temporaries[dp->first_args_temporary + j];
+                                    is_rw_operand[j] = 1;
+                                    rw_operands[j] = MVM_spesh_manipulate_get_temp_reg(tc, g, MVM_reg_uint64);
+                                    emit_bi_op(tc, g, bb, &insert_after, MVM_OP_decont_u, rw_operands[j], var);
+
+                                    callsite = MVM_callsite_replace_positional(tc, callsite, j, MVM_CALLSITE_ARG_UINT);
                                     MVM_callsite_intern(tc, &callsite, 1, 0);
 
                                     if (has_return_value && var.reg.orig == rb_ins->operands[0].reg.orig) {
@@ -1226,6 +1280,12 @@ static int translate_dispatch_program(MVMThreadContext *tc, MVMSpeshGraph *g,
                                 default:
                                     MVM_spesh_graph_add_comment(tc, g, ins, "dispatch not compiled: unsupported NativeCall rw type %d",
                                                     arg_types[j - 1] & MVM_NATIVECALL_ARG_TYPE_MASK);
+                                    /* Clean up the inserted guards */
+                                    ins = ins->next;
+                                    while (ins && ins != orig_next) {
+                                        MVM_spesh_manipulate_delete_ins(tc, g, bb, ins);
+                                        ins = ins->next;
+                                    }
                                     return 0;
                             }
                         }
@@ -1269,7 +1329,10 @@ static int translate_dispatch_program(MVMThreadContext *tc, MVMSpeshGraph *g,
                         MVMSpeshOperand var = has_rw_dummy
                             ? rw_dummy
                             : skip_args >= 0 ? args[skip_args + j] : temporaries[dp->first_args_temporary + j];
-                        emit_bi_op(tc, g, bb->linear_next, &post_call_instructions, MVM_OP_assign_i,
+                        emit_bi_op(tc, g, bb->linear_next, &post_call_instructions,
+                            g->local_types[rw_operands[j].reg.orig] == MVM_reg_int64
+                                ? MVM_OP_assign_i
+                                : MVM_OP_assign_u,
                             var, rw_operands[j]);
                         MVM_spesh_usages_add_by_reg(tc, g, var, post_call_instructions);
                         MVM_spesh_manipulate_release_temp_reg(tc, g, rw_operands[j]);
@@ -1363,6 +1426,9 @@ static int translate_dispatch_program(MVMThreadContext *tc, MVMSpeshGraph *g,
                     case MVM_OP_dispatch_i:
                         base_op = MVM_op_get_op(c ? MVM_OP_sp_runcfunc_i : MVM_OP_sp_runbytecode_i);
                         break;
+                    case MVM_OP_dispatch_u:
+                        base_op = MVM_op_get_op(c ? MVM_OP_sp_runcfunc_u : MVM_OP_sp_runbytecode_u);
+                        break;
                     case MVM_OP_dispatch_n:
                         base_op = MVM_op_get_op(c ? MVM_OP_sp_runcfunc_n : MVM_OP_sp_runbytecode_n);
                         break;
