@@ -24,13 +24,11 @@ static void demand_extop(MVMThreadContext *tc, MVMCompUnit *target_cu,
         if (extops[i].info == info) {
             MVMuint32 orig_size = target_cu->body.num_extops * sizeof(MVMExtOpRecord);
             MVMuint32 new_size = (target_cu->body.num_extops + 1) * sizeof(MVMExtOpRecord);
-            MVMExtOpRecord *new_extops = MVM_fixed_size_alloc(tc,
-                tc->instance->fsa, new_size);
+            MVMExtOpRecord *new_extops = MVM_malloc(new_size);
             memcpy(new_extops, target_cu->body.extops, orig_size);
             memcpy(&new_extops[target_cu->body.num_extops], &extops[i], sizeof(MVMExtOpRecord));
             if (target_cu->body.extops)
-                MVM_fixed_size_free_at_safepoint(tc, tc->instance->fsa, orig_size,
-                   target_cu->body.extops);
+                MVM_free_at_safepoint(tc, target_cu->body.extops);
             target_cu->body.extops = new_extops;
             target_cu->body.num_extops++;
             uv_mutex_unlock(target_cu->body.inline_tweak_mutex);
@@ -144,6 +142,7 @@ static int is_graph_inlineable(MVMThreadContext *tc, MVMSpeshGraph *inliner,
             if (!same_hll && runbytecode_ins->info->opcode == MVM_OP_sp_runbytecode_o) {
                 switch (opcode) {
                     case MVM_OP_return_i:
+                    case MVM_OP_return_u:
                     case MVM_OP_return_n:
                     case MVM_OP_return_s:
                         *no_inline_reason = "target needs a return boxing and HLLs are different";
@@ -1124,8 +1123,10 @@ static void return_to_box(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpeshBB *re
     target_facts = MVM_spesh_get_facts(tc, g, ver_target);
     target_facts->writer = box_ins;
     target_facts->flags |= MVM_SPESH_FACT_KNOWN_TYPE | MVM_SPESH_FACT_CONCRETE | MVM_SPESH_FACT_KNOWN_BOX_SRC;
+    // Note: we use hllboxtype_i for both box_i and box_u
     target_facts->type = box_op == MVM_OP_box_i ? g->sf->body.cu->body.hll_config->int_box_type :
                          box_op == MVM_OP_box_n ? g->sf->body.cu->body.hll_config->num_box_type :
+                         box_op == MVM_OP_box_u ? g->sf->body.cu->body.hll_config->int_box_type :
                                                   g->sf->body.cu->body.hll_config->str_box_type;
     MVM_spesh_usages_add_by_reg(tc, g, box_operands[1], box_ins);
     MVM_spesh_usages_add_by_reg(tc, g, box_operands[2], box_ins);
@@ -1158,6 +1159,25 @@ static void rewrite_int_return(MVMThreadContext *tc, MVMSpeshGraph *g,
             "Spesh inline: unhandled case (%s) of return_i", runbytecode_ins->info->name);
     }
 }
+static void rewrite_uint_return(MVMThreadContext *tc, MVMSpeshGraph *g,
+        MVMSpeshBB *return_bb, MVMSpeshIns *return_ins,
+        MVMSpeshBB *runbytecode_bb, MVMSpeshIns *runbytecode_ins) {
+    switch (runbytecode_ins->info->opcode) {
+    case MVM_OP_sp_runbytecode_v:
+        MVM_spesh_manipulate_delete_ins(tc, g, return_bb, return_ins);
+        break;
+    case MVM_OP_sp_runbytecode_u:
+        return_to_op(tc, g, return_ins, runbytecode_ins->operands[0], MVM_OP_set);
+        break;
+    case MVM_OP_sp_runbytecode_o:
+        return_to_box(tc, g, return_bb, return_ins, runbytecode_ins->operands[0],
+            MVM_OP_hllboxtype_i, MVM_OP_box_u);
+        break;
+    default:
+        MVM_oops(tc,
+            "Spesh inline: unhandled case (%s) of return_i", runbytecode_ins->info->name);
+    }
+}
 static void rewrite_num_return(MVMThreadContext *tc, MVMSpeshGraph *g,
                         MVMSpeshBB *return_bb, MVMSpeshIns *return_ins,
                         MVMSpeshBB *runbytecode_bb, MVMSpeshIns *runbytecode_ins) {
@@ -1262,6 +1282,14 @@ static void rewrite_returns(MVMThreadContext *tc, MVMSpeshGraph *inliner,
                 rewrite_int_return(tc, inliner, bb, ins, runbytecode_bb, runbytecode_ins);
                 saw_return = 1;
                 break;
+            case MVM_OP_return_u:
+                MVM_spesh_manipulate_insert_goto(tc, inliner, bb, ins,
+                    runbytecode_bb->succ[0]);
+                tweak_succ(tc, inliner, bb, runbytecode_bb, runbytecode_bb->succ[0],
+                    saw_return);
+                rewrite_uint_return(tc, inliner, bb, ins, runbytecode_bb, runbytecode_ins);
+                saw_return = 1;
+                break;
             case MVM_OP_return_n:
                 MVM_spesh_manipulate_insert_goto(tc, inliner, bb, ins,
                     runbytecode_bb->succ[0]);
