@@ -157,6 +157,18 @@ void MVM_spesh_log_type(MVMThreadContext *tc, MVMObject *value) {
     commit_entry(tc, sl);
 }
 
+void MVM_spesh_log_type_at(MVMThreadContext *tc, MVMObject *value, MVMuint8 *prev_op) {
+    MVMSpeshLog *sl = tc->spesh_log;
+    MVMint32 cid = tc->cur_frame->spesh_correlation_id;
+    MVMSpeshLogEntry *entry = &(sl->body.entries[sl->body.used]);
+    entry->kind = MVM_SPESH_LOG_TYPE;
+    entry->id = cid;
+    MVM_ASSIGN_REF(tc, &(sl->common.header), entry->type.type, value->st->WHAT);
+    entry->type.flags = IS_CONCRETE(value) ? MVM_SPESH_LOG_TYPE_FLAG_CONCRETE : 0;
+    entry->type.bytecode_offset = (prev_op - *(tc->interp_bytecode_start)) - 2;
+    commit_entry(tc, sl);
+}
+
 /* Log a decont, only those that did not invoke. */
 void MVM_spesh_log_decont(MVMThreadContext *tc, MVMuint8 *prev_op, MVMObject *value) {
     MVMSpeshLog *sl = tc->spesh_log;
