@@ -60,12 +60,7 @@ typedef struct MVMExtOpRegistry MVMExtOpRegistry;
 typedef struct MVMExtRegistry MVMExtRegistry;
 typedef struct MVMRegionAlloc MVMRegionAlloc;
 typedef struct MVMRegionBlock MVMRegionBlock;
-typedef struct MVMFixedSizeAlloc MVMFixedSizeAlloc;
-typedef struct MVMFixedSizeAllocFreeListEntry MVMFixedSizeAllocFreeListEntry;
-typedef struct MVMFixedSizeAllocSafepointFreeListEntry MVMFixedSizeAllocSafepointFreeListEntry;
-typedef struct MVMFixedSizeAllocSizeClass MVMFixedSizeAllocSizeClass;
-typedef struct MVMFixedSizeAllocThread MVMFixedSizeAllocThread;
-typedef struct MVMFixedSizeAllocThreadSizeClass MVMFixedSizeAllocThreadSizeClass;
+typedef struct MVMAllocSafepointFreeListEntry MVMAllocSafepointFreeListEntry;
 typedef struct MVMFrame MVMFrame;
 typedef struct MVMFrameExtra MVMFrameExtra;
 typedef struct MVMFrameHandler MVMFrameHandler;
